package nl.kallestruik.dtweaks.tweaks.miscellaneoustweaks

import net.kyori.adventure.text.Component
import net.kyori.adventure.text.format.NamedTextColor
import net.kyori.adventure.text.format.TextDecoration
import nl.kallestruik.dtweaks.DTweaks
import nl.kallestruik.dtweaks.tweaks.ITweak
import org.bukkit.Keyed
import org.bukkit.Material
import org.bukkit.NamespacedKey
import org.bukkit.attribute.Attribute
import org.bukkit.attribute.AttributeModifier
import org.bukkit.event.EventHandler
import org.bukkit.event.HandlerList
import org.bukkit.event.Listener
import org.bukkit.event.inventory.PrepareSmithingEvent
import org.bukkit.inventory.*
import org.bukkit.plugin.java.JavaPlugin
import java.util.*

class DyeableNetherite(
    private val plugin: DTweaks
): ITweak, Listener {
    override fun getIdentifier(): String = "DyeableNetherite"
    override fun getCategories(): List<String> = listOf("miscellaneous", "survival")

    private val helmetRecipeKey = NamespacedKey(plugin, "dyed_netherite_helmet")
    private val chestplateRecipeKey = NamespacedKey(plugin, "dyed_netherite_chestplate")
    private val leggingsRecipeKey = NamespacedKey(plugin, "dyed_netherite_leggings")
    private val bootsRecipeKey = NamespacedKey(plugin, "dyed_netherite_boots")

    private val helmetStats = ArmorStats(2.0, 3.0, 0.1, EquipmentSlot.HEAD)
    private val chestplateStats = ArmorStats(5.0, 3.0, 0.1, EquipmentSlot.CHEST)
    private val leggingsStats = ArmorStats(4.0, 3.0, 0.1, EquipmentSlot.LEGS)
    private val bootsStats = ArmorStats(2.0, 3.0, 0.1, EquipmentSlot.FEET)

    override fun onEnable() {
        plugin.server.pluginManager.registerEvents(this, plugin)

        addRecipe(helmetRecipeKey, Material.NETHERITE_HELMET, "Netherite Helmet", Material.LEATHER_HELMET)
        addRecipe(chestplateRecipeKey, Material.NETHERITE_CHESTPLATE, "Netherite Chestplate", Material.LEATHER_CHESTPLATE)
        addRecipe(leggingsRecipeKey, Material.NETHERITE_LEGGINGS, "Netherite Leggings", Material.LEATHER_LEGGINGS)
        addRecipe(bootsRecipeKey, Material.NETHERITE_BOOTS, "Netherite Boots", Material.LEATHER_BOOTS)
    }

    private fun addRecipe(key: NamespacedKey, sourceMaterial: Material, name: String, resultMaterial: Material) {
        val resultStack = ItemStack(resultMaterial)
        resultStack.editMeta { it.displayName(Component.text(name).color(NamedTextColor.WHITE).decoration(TextDecoration.ITALIC, false))}
        val recipe = SmithingRecipe(key, resultStack, RecipeChoice.MaterialChoice(sourceMaterial), RecipeChoice.MaterialChoice(Material.LEATHER))

        plugin.server.addRecipe(recipe)
    }

    override fun onDisable() {
        HandlerList.unregisterAll(this)
        plugin.server.removeRecipe(helmetRecipeKey)
        plugin.server.removeRecipe(chestplateRecipeKey)
        plugin.server.removeRecipe(leggingsRecipeKey)
        plugin.server.removeRecipe(bootsRecipeKey)
    }

    @EventHandler
    fun onSmith(event: PrepareSmithingEvent) {
        val recipe = event.inventory.recipe ?: return
        if (!isDyeableNetheriteRecipe(recipe))
            return

        val result = event.inventory.result ?: return

        when (result.type) {
            Material.LEATHER_HELMET -> helmetStats.addToItem(result)
            Material.LEATHER_CHESTPLATE -> chestplateStats.addToItem(result)
            Material.LEATHER_LEGGINGS -> leggingsStats.addToItem(result)
            Material.LEATHER_BOOTS -> bootsStats.addToItem(result)
            else -> return
        }

        event.inventory.result = result
    }

    private fun isDyeableNetheriteRecipe(recipe: Recipe): Boolean {
        if (recipe !is Keyed)
            return false

        if (recipe.key.namespace != plugin.getNamespace())
            return false

        if (!recipe.key.key.startsWith("dyed_netherite_"))
            return false

        return true
    }

    data class ArmorStats(
        val armor: Double,
        val toughness: Double,
        val knockbackResistance: Double,
        val slot: EquipmentSlot,
    ) {
        fun addToItem(item: ItemStack) {
            item.type = Material.DIAMOND
            item.editMeta {
                it.addAttributeModifier(Attribute.GENERIC_ARMOR, AttributeModifier(UUID.randomUUID(), "Armor", armor, AttributeModifier.Operation.ADD_NUMBER, slot))
                it.addAttributeModifier(Attribute.GENERIC_ARMOR_TOUGHNESS, AttributeModifier(UUID.randomUUID(), "Armor Toughness", toughness, AttributeModifier.Operation.ADD_NUMBER, slot))
                it.addAttributeModifier(Attribute.GENERIC_KNOCKBACK_RESISTANCE, AttributeModifier(UUID.randomUUID(), "Knockback Resistance", knockbackResistance, AttributeModifier.Operation.ADD_NUMBER, slot))
            }
        }
    }
}